package com.winhc.bigdata.spark.utils

import java.util.concurrent.{CountDownLatch, TimeUnit}

import org.apache.commons.lang3.time.StopWatch
import org.apache.spark.JobExecutionStatus
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

/**
 * @Author: XuJiakai
 * @Date: 2020/8/21 11:36
 * @Description:
 */
object AsyncExtract {

  def startAndWait(spark: SparkSession, seq: Seq[(String, () => Boolean)]): Unit = {
    start(seq)
    wait(spark)
  }

  def start(seq: Seq[(String, () => Boolean)]): Unit = {
    AsyncExtract(seq).start()
  }

  def wait(spark: SparkSession): Unit = {
    val tracker = spark.sparkContext.statusTracker
    var i = 0
    while (!tracker.getActiveJobIds().nonEmpty && i < 100) {
      i = i + 1
      println("await job 。。。")
      Thread.sleep(10000)
    }
    println(tracker.getActiveJobIds().mkString(","))
    while (tracker.getActiveJobIds().nonEmpty) {
      println(tracker.getActiveJobIds().mkString(","))
      println("spark is not stop ! ")
      Thread.sleep(10000)
    }

    val ints = tracker.getJobIdsForGroup(null)
    println(ints.mkString(","))
    val failed = ints
      .map(i => tracker.getJobInfo(i))
      .map(o => o.orNull)
      .filter(_ != null)
      .map(i => i.status()).exists(i => JobExecutionStatus.FAILED.equals(i))
    if (failed) {
      sys.exit(-999)
    }
  }
}

case class AsyncExtract(seq: Seq[(String, () => Boolean)]) extends Logging {

  def start(): Unit = {
    val latch = new CountDownLatch(seq.length)
    seq.foreach(e => {
      asyncWatch(e._1, () => {
        println("______________________________" + e._1 + "___________________________________")
        val res = e._2()
        println(s"---${e._1}---------$res-------------")
        latch.countDown()
      })
    })
    //    latch.await()
  }

  private def asyncWatch[T](name: String, f: () => Unit): Unit = {
    val t = new Thread(new Runnable {
      override def run(): Unit = {
        watch(name, f)
      }
    })
    t.setName(name)
    t.start()
  }

  private def watch[T](name: String, fun: () => T): T = {
    val stopWatch = new StopWatch
    stopWatch.start()
    println(
      s"""
         |
         |----------------------- submit [action=$name] begin... -----------------------
         |""".stripMargin)
    logInfo(
      s"""
         |
         |----------------------- submit [action=$name] begin... -----------------------
       """.stripMargin
    )

    val r = fun()
    stopWatch.stop()
    val totalSeconds = stopWatch.getTime(TimeUnit.SECONDS)

    println(
      s"""
         |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
         |
         |""".stripMargin)
    logInfo(
      s"""
         |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
         |
       """.stripMargin
    )
    r
  }

  private def toTimeStr(totalSeconds: Long): String = {
    val hours = TimeUnit.SECONDS.toHours(totalSeconds)
    val minutes = TimeUnit.SECONDS.toMinutes(totalSeconds - hours * 3600)
    val seconds = totalSeconds - hours * 3600 - minutes * 60
    val timeStr = if (hours > 0) {
      s"${hours}h${minutes}m${seconds}s"
    } else if (minutes > 0) {
      s"${minutes}m${seconds}s"
    } else {
      s"${seconds}s"
    }
    timeStr
  }
}
